/******************************************************************************
 *                                                                            *
 * File:   nbmac.c            Version  2.05            Date: 2010-04-03       *
 *                                                                            *
 * Copyright (C) 1998-2010 by Kostis Netzwerkberatung                         *
 * Talstr. 25, D-63322 Rdermark, Tel. +49 6074 881056                        *
 * http://www.kostis.net/                                                     *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * Function:                                                                  *
 *        - display MAC address using NetBIOS                                 *
 *                                                                            *
 ******************************************************************************
 *                                                                            *
 * History:                                                                   *
 *     2010-04-03: KK V2.05                                                   *
 *        - recompiled after corrections in netbios.h/.lib                    *
 *     2010-03-14: KK V2.04                                                   *
 *        - added upper case option                                           *
 *          sugguested by Wouter Rozendaal                                    *
 *     2009-03-07: KK V2.03                                                   *
 *        - internal cosmetic changes                                         *
 *        - moved ScanFlag () into main source file                           *
 *     2009-03-04: KK V2.02                                                   *
 *        - updated syntax message                                            *
 *     2009-02-27: KK V2.01                                                   *
 *        - recompiled after fixing bug in netbios.lib StrMAC function        *
 *     2009-02-26: KK V2.00                                                   *
 *        - added -b, -c, -i, -l, -r, -s                                      *
 *     1998-11-06: KK V1.03                                                   *
 *        - lana_num now from 0 to 255                                        *
 *          sugguested by Lee Meador                                          *
 *     1998-05-04: KK V1.02                                                   *
 *        - changed MAC address output format                                 *
 *     1998-04-05: KK V1.01                                                   *
 *        - made NetBIOS library more compliant to nb30.h                     *
 *     1998-03-28: KK V1.00                                                   *
 *        - initial coding                                                    *
 *                                                                            *
 *****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <netbios.h>

#include "nbmac.h"

/******************************************************************************

	Function:
		display copyright message and program information

	Parameter:
		FILE		*Fout		stdout oder stderr

	Returns:
		nothing

 *****************************************************************************/

static  void    Hello
(
	FILE		*Fout
)
{
	fprintf (Fout, MSG_COPYRIGHT) ;
	fprintf (Fout, MSG_VERSION, PRGNAM, VERSION) ;
}

/******************************************************************************

	Function Variable Values

 *****************************************************************************/

#define	SCANFLAG_YES		1
#define	SCANFLAG_NO		0

/******************************************************************************

	Function Return Values

 *****************************************************************************/

#define SCANFLAG_OK		0	/* options all valid		*/
#define SCANFLAG_UNKNOWN	1	/* unknown option		*/
#define SCANFLAG_BADARGS	2	/* illegal option (value)	*/
#define SCANFLAG_MULTIPLE	3	/* multiple exclusive options	*/

/******************************************************************************

	Internal Constants about MAC format and case

 *****************************************************************************/

#define	MAC_CASE_DEFAULT	0
#define	MAC_CASE_UPPER		1

/******************************************************************************

	Global Variables

 *****************************************************************************/

char	*nbname	= (char *) NULL ;
int	lana	= 0 ;
int	batch	= SCANFLAG_NO ;
int	scan	= SCANFLAG_NO ;
int	format	= MAC_FORMAT_DEFAULT ;
int	mcase	= MAC_CASE_DEFAULT ;

/******************************************************************************

	Function:
		parse parameters

	Parameter:
		int		argc		# of parameters
		char		*argv	[]	parameters

	Returns:
		SCANFLAG_OK			all parameters valid

		SCANFLAG_UNKNOWN		unknown parameter
		SCANFLAG_BADARGS		illegal parameter(number)

 *****************************************************************************/

int	ScanFlag
(
	int		argc,
	char		*argv	[]
)
{
	int		i ;
	int		parms		= 0 ;	/* yet no parameter scanned  */

	/**********************************************************************

		set defaults

	 *********************************************************************/


	for (i = 1 ; i < argc ; ++i)	/* scan options */
	{
		/**************************************************************

			is it a flag/switch (*NIX style)

		 *************************************************************/

		if ((*argv [i] == '-') || (*argv [i] == '/'))
		{
			/******************************************************

				which flag?

			 *****************************************************/

			switch (argv [i][1])
			{
				case	'0':
				case	'1':
				case	'2':
				case	'3':
				case	'4':
				case	'5':
				case	'6':
				case	'7':
				case	'8':
				case	'9':

					lana = atoi (&argv [i][1]) ;
					if ((lana > 255) || (lana < 0))
						return (SCANFLAG_BADARGS) ;
					break ;

				case	'b':
					batch = SCANFLAG_YES ;
					break ;

				case	's':
					scan = SCANFLAG_YES ;
					break ;

				case	'i':

					if (format != MAC_FORMAT_DEFAULT)
					{
						return (SCANFLAG_MULTIPLE) ;
					}
					format = MAC_FORMAT_IEEE ;
					break ;

				case	'c':

					if (format != MAC_FORMAT_DEFAULT)
					{
						return (SCANFLAG_MULTIPLE) ;
					}
					format = MAC_FORMAT_CISCO ;
					break ;

				case	'l':

					if (format != MAC_FORMAT_DEFAULT)
					{
						return (SCANFLAG_MULTIPLE) ;
					}
					format = MAC_FORMAT_COLON ;
					break ;

				case	'r':

					if (format != MAC_FORMAT_DEFAULT)
					{
						return (SCANFLAG_MULTIPLE) ;
					}
					format = MAC_FORMAT_RAW ;
					break ;

				case	'u':

					mcase = MAC_CASE_UPPER ;
					break ;

				default:        /* unknown flag		*/

					return (SCANFLAG_UNKNOWN) ;
			}
		}
		else
		{
			/******************************************************

				no flag, got to be a nbname

			 *****************************************************/

			++parms ;	/* copy	nbname */
			switch (parms)
			{
				case	1:

					nbname = argv [i] ;
					break ;

				default :

					return (SCANFLAG_BADARGS) ;
			}
		}
	}

	if (format == MAC_FORMAT_DEFAULT)
	{
		format = MAC_FORMAT_IEEE ;
	}

	return (SCANFLAG_OK) ;
}

/******************************************************************************

	main programm

 *****************************************************************************/

int	main
(
	int		argc,
	char		*argv	[]
)
{
	PNCB		pncb ;
	PADAPTER_STATUS	pADAPTER_STATUS ;
	int		i ;
	int		rc ;
	int		lana_e ;
	char		sMAC [MAC_MAX_STRLEN + 1] ;

	/**********************************************************************

		check syntax

	 *********************************************************************/

	if (ScanFlag (argc, argv) != SCANFLAG_OK)
	{
		Hello (stdout) ;
		fprintf (stdout, ERR_MSG_SYNTAX, PRGNAM) ;
		return (ERR_SYNTAX) ;
	}

	if (nbname != (char *) NULL)
	{
		rc = CheckNBName (nbname) ;
		if (rc != NBCHK_OK)
		{
			switch (rc)
			{
				case	NBCHK_NAME_TOOLONG:

					fprintf (stdout,
						ERR_MSG_NAME_TOOLONG, PRGNAM) ;
					return (ERR_NAME_TOOLONG) ;

				case	NBCHK_NAME_INVALID:

					fprintf (stdout,
						ERR_MSG_NAME_INVALID, PRGNAM) ;
					return (ERR_NAME_INVALID) ;
			}
		}
	}
	else
	{
		nbname = "*" ;
	}

	/**********************************************************************

		fetch memory for data structures

	 *********************************************************************/

	pncb = (PNCB) malloc (sizeof (NCB)) ;
	if (pncb == (PNCB) NULL)
		return (ERR_OUT_OF_MEMORY) ;

	pADAPTER_STATUS = (PADAPTER_STATUS) malloc (sizeof (ADAPTER_ALL)) ;
	if (pADAPTER_STATUS == (PADAPTER_STATUS) NULL)
		return (ERR_OUT_OF_MEMORY) ;

	/**********************************************************************

		initialize / setup data structures

	 *********************************************************************/

	lana_e = lana ;
	if (scan == SCANFLAG_YES)
	{
		lana_e = 255 ;
	}

	ClearNCB (pncb) ;
	ClearADAPTER_STATUS (pADAPTER_STATUS) ;

	do
	{
		pncb -> ncb_command  = NCBASTAT ;
		_fmemcpy (pncb -> ncb_callname, nbname, strlen (nbname)) ;
		pncb -> ncb_lana_num = lana ;
		pncb -> ncb_buffer   = (PUCHAR) pADAPTER_STATUS ;
		pncb -> ncb_length   = sizeof (ADAPTER_ALL) ;

		/**************************************************************

			make NetBIOS call and check results

		 *************************************************************/

		Netbios (pncb) ;

		if (pncb -> ncb_retcode != NRC_GOODRET)
		{
			++lana ;
		}
	}
	while ((pncb -> ncb_retcode != NRC_GOODRET) && (lana <= lana_e)) ;

	/**********************************************************************

		if NetBIOS call was successful, display result

	 *********************************************************************/
	
	if (pncb -> ncb_retcode == NRC_GOODRET)
	{
		StrMAC (pADAPTER_STATUS -> adapter_address, sMAC, format) ;

		if (batch == SCANFLAG_YES)
		{
			printf ("set NBMAC=") ;
		}

		if (mcase == MAC_CASE_UPPER)
		{
			strupr (sMAC) ;
		}

		printf ("%s\n", sMAC) ;
	}


	/**********************************************************************

		free memory for data structures

	 *********************************************************************/

	free ((void *) pADAPTER_STATUS) ;
	free ((void *) pncb) ;

	return (ERR_NONE) ;
}
